function VarTypeReference(
    table,      // Responsible TableOfVars instance
    varName,    // Name of this variable
    varType,    // Name of (base) type
    dimensions, // Optional: Number of dimensions (default 0, greater 0 means array)
    abstract    // Optional: True if new is disabled
) {
    this.baseType = varType;
    if (abstract)
        this._abstract = true;
    if (dimensions)
        this.dimensions = dimensions;
    VarType.call(this, table, varName);
}
VarTypeReference.prototype = Object.create(VarType.prototype);
VarTypeReference.prototype.constructor = VarTypeReference;

/** Used number of bytes to store this variable */
VarTypeReference.countBytes = 4;

/**
* Add table rows to this.table
* @return undefined
*/
VarTypeReference.prototype.createTr = function() {
    this._tr = this.table.addTrVar(this.varName, this.table.createCellContentType(this.table.getDocument(), this.baseType, this.dimensions), this.input, () => this.remove());
}

/** Name of variable's base type */
VarTypeReference.prototype.baseType = '';

/** Number of dimensions (default 0, greater 0 means array) */
VarTypeReference.prototype.dimensions = 0;

/** Name of variable type */
VarTypeReference.typeName = null;
Object.defineProperty(VarTypeReference.prototype, 'typeName', {
    enumerable: true,
    get() { return this.baseType + '[]'.repeat(this.dimensions); }
});

/** Default value of variables of this type */
VarTypeReference.defaultValue = null;

/** Present value */
Object.defineProperty(VarTypeReference.prototype, 'value', { enumerable: true,
    get() { return this.input.options[Math.max(0, this.input.selectedIndex)].textContent; },
    set(v) {
        const index = (v == 'null') ? 0 : (Array.from(this.input.options).map(node => node.textContent).slice(2).indexOf(v) + 2);
        this.input.selectedIndex = (index < 2) ? 0 : index;
    }
});

/**
* Create a input for this type
* @return HTMLElement       Created input, select, …
*/
VarTypeReference.prototype._createInput = function(
    document        // HTMLDocument instance to use
) {
    const input = document.createElement('select');
    ScoTools.createHtml('option', input, null, 'null');
    ScoTools.createHtml('option', input, this._abstract ? { disabled: 'disabled' } : null, 'new');
    this.table.getInstancesOf(this.baseType, this.dimensions).forEach(address => ScoTools.createHtml('option', input, null, address));
    input.selectedIndex = 0;
    input.addEventListener('change', () => {
        if (input.selectedIndex != 1)
            return;
        const created = this.table.createInstancesOf(this.baseType, this.dimensions);
        const index = Array.from(input.options).slice(2).map(option => option.textContent).indexOf(created) + 2;
        input.selectedIndex = (index > 1) ? index : 0;
    });
    return input;
}

/**
* Add another instance to list of possible values
* @return undefined
*/
VarTypeReference.prototype.addInstance = function(
    address     // Human readable address of instance to add
) {
    if (Array.from(this.input.options).slice(2).filter(({ text }) => (text == address)).length)
        return;
    const value = this.value;
    const insertAtIndex = Array.from(this.input.options).slice(2).findIndex(option => (address < option.textContent)) + 2;
    if (insertAtIndex == 1)
        ScoTools.createHtml('option', this.input, null, address);
    else
        ScoTools.createHtml('option', this.input, null, address, this.input.childNodes[insertAtIndex]);
    this.input.selectedIndex = Math.max(0, Array.from(this.input.options).map(option => option.textContent).indexOf(value));
}

/**
* Remove an instance to list of possible values
* @return undefined
*/
VarTypeReference.prototype.removeInstance = function(
    address     // Human readable address of instance to remove
) {
    const value = this.value;
    Array.from(this.input.options).filter(option => (option.textContent == address)).forEach(option => option.parentNode.removeChild(option));
    this.input.selectedIndex = Math.max(0, Array.from(this.input.options).map(option => option.textContent).indexOf(value));
}

/**
* Convert contained data to JSON
* @return Object        JSON representation of contained data
*/
VarTypeReference.prototype.toJSON = function() {
    const json = {
        key: this.varName,
        type: this.baseType,
        ref: this.table.addressToPointer(this.value)
    };
    if (this.dimensions)
        json.dimensions = this.dimensions;
    return json;
}
